from django.db import models

from notifications.enums import EventType, ServiceType


class Notification(models.Model):
    """Generic/Abstract Poll with common fields used for Inheritance"""
    icon = models.PositiveSmallIntegerField(verbose_name='Icon')
    sub_icon = models.PositiveSmallIntegerField(verbose_name='Sub icon')
    event_creator = models.PositiveSmallIntegerField(choices=ServiceType.choices, 
                                                    default=ServiceType.SERVICE1, verbose_name='Creator')
    title = models.CharField(max_length=28, verbose_name='Title')
    context = models.TextField(max_length=95, verbose_name='Description')
    event_type = models.PositiveSmallIntegerField(choices=EventType.choices, default=EventType.CREATE, verbose_name='Type')
    dynamic_link = models.CharField(max_length=256, verbose_name='Dynamic link')
    viewed = models.BooleanField(default=False, verbose_name='Viewed')
    valid = models.BooleanField(default=True, verbose_name='Valid')
    valid_until = models.DateTimeField(null=True, verbose_name='Valid until timestamp')
    clicked = models.BooleanField(default=False, verbose_name='Clicked')
    clicked_at = models.DateTimeField(null=True, verbose_name='Clicked at timestamp')
    location = models.CharField(null=True, default=None, max_length=32, verbose_name='Location')
    channel = models.CharField(null=True, default=None, max_length=32, verbose_name='Channel')
    status = models.CharField(default='Not Published', max_length=16, verbose_name='Status')
    routing_key = models.CharField(max_length=32, verbose_name='Routing Key')

    def __str__(self):
        return f"{self.pk} | {self.event_creator} | {self.title}"



from django.contrib import admin
from django.urls import path
from notifications.views import NotificationListCreate, NotificationRetrieveUpdateDestroyAPIView


urlpatterns = [
    path("", NotificationListCreate.as_view(), name="notification-list-create"),
    path("<int:pk>", NotificationRetrieveUpdateDestroyAPIView.as_view(), name="notification-retrieve-update-destroy"),
]



from pika.exchange_type import ExchangeType
import json


class Publisher:

    def __init__(self, connection):
        self.connection = connection
        self.channel = connection.channel

    def publish(self, data):
        self.channel.exchange_declare(exchange='topicexchange', exchange_type=ExchangeType.topic)
        self.channel.basic_publish(exchange='topicexchange', routing_key=data["routing_key"], body=json.dumps(data, default=str))
        print(f"sent message: {data}")



import time
import json
from pika.exchange_type import ExchangeType
from abc import ABC, abstractmethod


class Consumer(ABC):

    def __init__(self, connection):
        self.connection = connection
        self.channel = connection.channel

    def consume(self):
        pass

    def on_message_received(self, ch, method, properties, body):
        pass


class Common:

    def __init__(self, connection):
        self.connection = connection
        self.channel = connection.channel

    def consume(self):
        self.channel.exchange_declare(exchange='topicexchange', exchange_type=ExchangeType.topic)
        queue = self.channel.queue_declare(queue='', exclusive=True)

        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='#.common')
        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='all')

        self.channel.basic_consume(queue=queue.method.queue, auto_ack=False,
            on_message_callback=self.on_message_received)

        print(f"Common Starts Consuming...")

        self.channel.start_consuming()

    def on_message_received(self, ch, method, properties, body):
        print(f"Common received new message: {json.loads(body)}")
        ch.basic_ack(delivery_tag=method.delivery_tag)
        print(f"Common finished processing and acknowledged message")


class ChatIt:

    def __init__(self, connection):
        self.connection = connection
        self.channel = connection.channel

    def consume(self):
        self.channel.exchange_declare(exchange='topicexchange', exchange_type=ExchangeType.topic)
        queue = self.channel.queue_declare(queue='', exclusive=True)

        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='#.chatit')
        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='all')

        self.channel.basic_consume(queue=queue.method.queue, auto_ack=False,
            on_message_callback=self.on_message_received)

        print(f"ChatIt Starts Consuming...")

        self.channel.start_consuming()

    def on_message_received(self, ch, method, properties, body):
        print(f"ChatIt received new message: {json.loads(body)}")
        ch.basic_ack(delivery_tag=method.delivery_tag)
        print(f"ChatIt finished processing and acknowledged message")


class PollIt:

    def __init__(self, connection):
        self.connection = connection
        self.channel = connection.channel

    def consume(self):
        self.channel.exchange_declare(exchange='topicexchange', exchange_type=ExchangeType.topic)
        queue = self.channel.queue_declare(queue='', exclusive=True)

        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='#.pollit')
        self.channel.queue_bind(exchange='topicexchange', queue=queue.method.queue, routing_key='all')

        self.channel.basic_consume(queue=queue.method.queue, auto_ack=False,
            on_message_callback=self.on_message_received)

        print(f"PollIt Starts Consuming...")

        self.channel.start_consuming()

    def on_message_received(self, ch, method, properties, body):
        print(f"PollIt received new message: {json.loads(body)}")
        ch.basic_ack(delivery_tag=method.delivery_tag)
        print(f"PollIt finished processing and acknowledged message")



from django.utils.translation import gettext_lazy as _
from django.db.models import IntegerChoices


class EventType(IntegerChoices):
    """Notification Event Type"""
    CREATE = 1, _('Create New Notification')
    UPDATE = 2, _('Update Existing Notification')



import pika
import json

HEARTBEAT = 60
BLOCKED_CONNECTION_TIMEOUT = 200


class SingletonMetaclass(type):

    _instance = None

    def __call__(cls, *args, **kwargs):
        assert not (args or kwargs), 'Singleton should not accept arguments'

        if isinstance(cls._instance, cls):
            return cls._instance
        else:
            cls._instance = super(SingletonMetaclass, cls).__call__()
            return cls._instance


class Connection(metaclass=SingletonMetaclass):

    def __init__(self):
        print("Creating new connection..")
        self.parameters = pika.ConnectionParameters('localhost', heartbeat=HEARTBEAT, \
                                                    blocked_connection_timeout=BLOCKED_CONNECTION_TIMEOUT)
        self.connection = pika.BlockingConnection(self.parameters)
        self.channel = self.connection.channel()

    def reconnect(self):
        print("Reconnection..")
        self.__init__()



import pika
from rest_framework import serializers

from notifications.connection import Connection
from notifications.models import Notification
from notifications.producer import Publisher


class NotificationSerializer(serializers.ModelSerializer):

    status = serializers.PrimaryKeyRelatedField(read_only=True)

    class Meta:
        model = Notification
        fields = '__all__'

    def create(self, validated_data):
        instance = super().create(validated_data)

        connection = Connection()
        publisher = Publisher(connection)
        try:
            publisher.publish(validated_data)
        except (pika.exceptions.StreamLostError, pika.exceptions.ChannelWrongStateError) as e:
            connection.reconnect()
            publisher.channel = connection.channel
            publisher.publish(validated_data)
        except Exception as e:
            return instance

        instance.status = "Published"
        instance.save()
        return instance

    # def update(self, instance, validated_data):
    #     instance = super().update(instance, validated_data)
    #     instance.status = "Not Published"

    #     connection = Connection()
    #     publisher = Publisher(connection)
    #     try:
    #         publisher.publish(instance)
    #     except (pika.exceptions.StreamLostError, pika.exceptions.ChannelWrongStateError) as e:
    #         connection.reconnect()
    #         publisher.channel = connection.channel
    #         publisher.publish(instance)

    #     instance.status = "Published"
    #     return instance



from django.utils.translation import gettext_lazy as _
from django.db.models import IntegerChoices


class ServiceType(IntegerChoices):
    """Notification Service Type"""
    SERVICE1 = 1, _('SERVICE1')
    SERVICE2 = 2, _('SERVICE2')
    SERVICE3 = 3, _('SERVICE3')
    SERVICE4 = 4, _('SERVICE4')



from django.apps import AppConfig


class NotificationsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'notifications'


# List get* 
# Filters date 
# Filters for service
# Storing db
# Tables for Services; Available Notification Types; User preferences; Users; Notifications; 
# Reusable Django App; INSTALLED_APPS setting;
# bubbling-firebase-authentication; 
# every app on shared/ not the bubbling-default-service
# pulling timed notifications from the dB to put in a queue



# Generated by Django 3.2.11 on 2022-12-30 06:07

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('icon', models.PositiveSmallIntegerField(verbose_name='Icon')),
                ('sub_icon', models.PositiveSmallIntegerField(verbose_name='Sub icon')),
                ('event_creator', models.PositiveSmallIntegerField(choices=[(1, 'SERVICE1'), (2, 'SERVICE2'), (3, 'SERVICE3'), (4, 'SERVICE4')], default=1, verbose_name='Creator')),
                ('title', models.CharField(max_length=28, verbose_name='Title')),
                ('context', models.TextField(max_length=95, verbose_name='Description')),
                ('event_type', models.PositiveSmallIntegerField(choices=[(1, 'Create New Notification'), (2, 'Update Existing Notification')], default=1, verbose_name='Type')),
                ('dynamic_link', models.CharField(max_length=256, verbose_name='Dynamic link')),
                ('viewed', models.BooleanField(default=False, verbose_name='Viewed')),
                ('valid', models.BooleanField(default=True, verbose_name='Valid')),
                ('valid_until', models.DateTimeField(null=True, verbose_name='Valid until timestamp')),
                ('clicked', models.BooleanField(default=False, verbose_name='Clicked')),
                ('clicked_at', models.DateTimeField(null=True, verbose_name='Clicked at timestamp')),
                ('location', models.CharField(default=None, max_length=32, null=True, verbose_name='Location')),
                ('channel', models.CharField(default=None, max_length=32, null=True, verbose_name='Channel')),
                ('status', models.CharField(default='Not Published', max_length=16, verbose_name='Status')),
                ('routing_key', models.CharField(max_length=32, verbose_name='Routing Key')),
            ],
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from notifications.consumers import Common
from notifications.connection import Connection
import pika


connection = Connection()
common_consumer = Common(connection)

class Command(BaseCommand):
    help = 'This command will run all Consumers'

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):

        try:
            common_consumer.consume()
        except (pika.exceptions.StreamLostError, pika.exceptions.ChannelWrongStateError) as e:
            connection.reconnect()
            common_consumer.channel = connection.channel
            common_consumer.consume()



import time

import pika
from django.shortcuts import render
from rest_framework import generics
from rest_framework.response import Response

from notifications.connection import Connection
from notifications.producer import Publisher
from notifications.serializers import NotificationSerializer
from notifications.models import Notification


class NotificationListCreate(generics.ListCreateAPIView):
    queryset = Notification.objects.all()
    serializer_class = NotificationSerializer
    pagination_class = None
    permission_classes = []


class NotificationRetrieveUpdateDestroyAPIView(generics.RetrieveUpdateDestroyAPIView):
    queryset = Notification.objects.all()
    serializer_class = NotificationSerializer
    pagination_class = None
    permission_classes = []
